@@ -52,18 +52,26 @@ def quad(func, a, b, args=(), full_output=0, epsabs=1.49e-8, epsrel=1.49e-8,
 
     Parameters
     ----------
-    func : function
+    func : {function, scipy.LowLevelCallable}
         A Python function or method to integrate.  If `func` takes many
         arguments, it is integrated along the axis corresponding to the
         first argument.
-        If the user desires improved integration performance, then f may
-        instead be a ``ctypes`` function of the form:
 
-            f(int n, double args[n]),
+        If the user desires improved integration performance, then `f` may
+        be a `scipy.LowLevelCallable` with one of the signatures::
 
-        where ``args`` is an array of function arguments and ``n`` is the
-        length of ``args``. ``f.argtypes`` should be set to
-        ``(c_int, c_double)``, and ``f.restype`` should be ``(c_double,)``.
+            double func(double x)
+            double func(double x, void *user_data)
+            double func(int n, double *xx)
+            double func(int n, double *xx, void *user_data)
+
+        The ``user_data`` is the data contained in the `scipy.LowLevelCallable`.
+        In the call forms with ``xx``,  ``n`` is the length of the ``xx`` 
+        array which contains ``xx[0] == x`` and the rest of the items are
+        numbers contained in the ``args`` argument of quad.
+
+        In addition, certain ctypes call signatures are supported for 
+        backward compatibility, but those should not be used in new code.
     a : float
         Lower limit of integration (use -numpy.inf for -infinity).
     b : float
@@ -492,7 +500,10 @@ def dblquad(func, a, b, gfun, hfun, args=(), epsabs=1.49e-8, epsrel=1.49e-8):
     scipy.special : for coefficients and roots of orthogonal polynomials
 
     """
-    return nquad(func, [lambda x: [gfun(x), hfun(x)], [a, b]], args=args)
+    def temp_ranges(*args):
+        return [gfun(args[0]), hfun(args[0])]
+    return nquad(func, [temp_ranges, [a, b]], args=args, 
+            opts={"epsabs": epsabs, "epsrel": epsrel})
 
 
 def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
@@ -564,7 +575,8 @@ def tplquad(func, a, b, gfun, hfun, qfun, rfun, args=(), epsabs=1.49e-8,
         return [gfun(args[0]), hfun(args[0])]
 
     ranges = [ranges0, ranges1, [a, b]]
-    return nquad(func, ranges, args=args)
+    return nquad(func, ranges, args=args, 
+            opts={"epsabs": epsabs, "epsrel": epsrel})
 
 
 def nquad(func, ranges, args=None, opts=None, full_output=False):
@@ -578,27 +590,24 @@ def nquad(func, ranges, args=None, opts=None, full_output=False):
 
     Parameters
     ----------
-    func : callable
+    func : {callable, scipy.LowLevelCallable}
         The function to be integrated. Has arguments of ``x0, ... xn``,
         ``t0, tm``, where integration is carried out over ``x0, ... xn``, which
         must be floats.  Function signature should be
         ``func(x0, x1, ..., xn, t0, t1, ..., tm)``.  Integration is carried out
         in order.  That is, integration over ``x0`` is the innermost integral,
         and ``xn`` is the outermost.
-        If performance is a concern, this function may be a ctypes function of
-        the form::
 
-            f(int n, double args[n])
+        If the user desires improved integration performance, then `f` may
+        be a `scipy.LowLevelCallable` with one of the signatures::
+
+            double func(int n, double *xx)
+            double func(int n, double *xx, void *user_data)
 
         where ``n`` is the number of extra parameters and args is an array
-        of doubles of the additional parameters.  This function may then
-        be compiled to a dynamic/shared library then imported through
-        ``ctypes``, setting the function's argtypes to ``(c_int, c_double)``,
-        and the function's restype to ``(c_double)``.  Its pointer may then be
-        passed into `nquad` normally.
-        This allows the underlying Fortran library to evaluate the function in
-        the innermost integration calls without callbacks to Python, and also
-        speeds up the evaluation of the function itself.
+        of doubles of the additional parameters, the ``xx`` array contains the 
+        coordinates. The ``user_data`` is the data contained in the
+        `scipy.LowLevelCallable`.
     ranges : iterable object
         Each element of ranges may be either a sequence  of 2 numbers, or else
         a callable that returns such a sequence.  ``ranges[0]`` corresponds to
